import Globals, time, string, Products, sys
from DateTime import DateTime
from Globals import Persistent, Acquisition
import AccessControl
from OFS.Folder import Folder
from OFS.Cache import Cacheable
from AccessControl import ClassSecurityInfo
try:
  from stripogram import html2text, html2safehtml
  _no_stripogram = 0
except:
  from Products.PythonScripts.standard import html_quote
  _no_stripogram = 1
  pass
from zLOG import format_exception, LOG, ERROR, INFO
from DocumentTemplate.DT_HTML import HTML as DTML
from StructuredText.StructuredText import HTML as STX
from Products.PageTemplates.PageTemplate import PageTemplate as PT
from XWorkFlow import XWorkFlow
from Catalogavel import Catalogavel
from ZCoMIX_Common import ZCoMIX_Common
from Categorizavel import Categorizavel
from PropriedadesGerenciaveis import PropriedadesGerenciaveis
from MultiCatalogAware.MultiCatalogPathAwareness import MultiCatalogAware

_marker = []
refresh_exc_info = {}

class XConteudo (Catalogavel, Categorizavel, PropriedadesGerenciaveis, XWorkFlow, ZCoMIX_Common, PT, Cacheable, Folder):
    """Classe que define o XConteudo."""

    friendly_name = 'Contedo'
    id_prox = ""
    data_inicial = DateTime('2001/01/01')
    data_final = DateTime('2030/01/01')
    mostrar_no_menu_superior = 0
    ordem_menu_superior = 0
    mostrar_no_menu_esquerdo = 1
    ordem_menu_esquerdo = 0
    mostrar_no_menu_direito = 0
    ordem_menu_direito = 0
    _text = ""
    categorias = None
    _document_format = 'DTML'
    _document_formats = (
			{'id':'HTML', 'label':'HTML'},
			{'id':'PLAIN', 'label':'Texto sem formatao'},
			{'id':'DTML', 'label':'Document Template'},
			{'id':'STX', 'label':'Texto Estruturado'},
			{'id':'ZPT', 'label':'Zope Page Template'},
			)

    # Compatibilidade com versoes anteriores
    mostrar_no_menu = mostrar_no_menu_esquerdo
    mostrar_no_menu_principal = mostrar_no_menu_superior
    ordem_menu_principal = ordem_menu_superior
    ordem_menu_secao = ordem_menu_esquerdo

    security = ClassSecurityInfo()

    security.declareProtected('View management screens', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXPaginaForm", globals(), in_ZMI=1)

    security.declareProtected('View management screens', 'manage_IEEditorForm')
    manage_IEEditorForm = Globals.DTMLFile("dtml/editIEEditorForm", globals(), in_ZMI=1)

    security.declareProtected('View','errorStatusPublicacao')
    errorStatusPublicacao = Globals.DTMLFile("dtml/errorStatusPublicacao", globals())

    security.declareProtected('View','errorTemplatePrincipal')
    errorTemplatePrincipal = Globals.DTMLFile("dtml/errorTemplatePrincipal", globals())

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'produtor','type':'string','mode':'w'},
        {'id':'data_producao','type':'date','mode':'w' },
        {'id':'revisor','type':'string','mode':'w'},
        {'id':'data_revisao','type':'date','mode':'w' },
        {'id':'publicador','type':'string','mode':'w'},
        {'id':'data_publicacao','type':'date','mode':'w' },
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        )

    _cache_namespace_keys=()

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in ('DTML Method', 'DTML Document', 'Image', 'Z SQL Method',  'Script (Python)',
                                                'Folder', 'File', 'Formulator Form', 'XLink')
        return filter(f, Products.meta_types)

    security.declareProtected('Access contents information', 'getTexto')
    def getTexto(self, REQUEST=None):
        "Retorna o conteudo do objeto como Texto."
        result = ""
        if hasattr(self, '_text'):
          result = "%s" % self._text
        return result

    security.declareProtected('Access contents information', 'getSize')
    def getSize(self, REQUEST=None):
        "Retorna o tamanho do objeto."
        result = 0
        if hasattr(self, '_text'):
          result = len(self._text)
        return result

    get_size = getSize

    def __str__(self, REQUEST=None):
        "Retorna o conteudo do objeto como Texto."
        return self.getTexto()

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None):
        """ Visualizacao do Conteudo """
        if hasattr(self, 'getTemplate'):
          tmpl = self.getTemplate()
        else:
          tmpl = self.getTemplatePrincipal()
        if tmpl is None:
          return self.errorTemplatePrincipal(REQUEST)
        tmpl = getattr(self, tmpl)
        index = getattr(tmpl.aq_base, 'index_html', None)
        if index is None:
          return self.errorTemplatePrincipal(REQUEST)
        #index = index.aq_base
        if hasattr(index, '__of__'):
                index = index.__of__(self)
                return index(self, REQUEST)

    security.declareProtected('Access contents information', 'getDocumentFormat')
    def getDocumentFormat(self, REQUEST=None):
        "Retorna o tipo de formatao selecionada para o objeto."
        if hasattr(self, '_usar_structured_text'):
          if self._usar_structured_text:
            return 'STX'
        if hasattr(self, '_document_format'):
          return self._document_format
        return 'DTML'

    security.declareProtected('Access contents information', 'getDocumentFormats')
    def getDocumentFormats(self, REQUEST=None):
        "Retorna o tipo de formatao disponivel para o objeto."
        if hasattr(self, '_document_formats'):
          return self._document_formats
        return None

    def manage_options(self, REQUEST=None):
        options=((
            {'label':'Propriedades', 'action':'manage_PropertiesForm', 'help':('ZCoMIX','Conteudo_Propriedades.stx')},
            {'label':'Workflow', 'action':'manage_WorkFlowForm','help':('ZCoMIX','XWorkflow_Workflow.stx')},
            {'label':'Visualizar', 'action':''},
            )
            #+ MultiCatalogAware.manage_options
            + Folder.manage_options[:1]
            + Folder.manage_options[3:]
            + Cacheable.manage_options
            )
        if self.getDocumentFormat() == 'HTML' and string.find(self.REQUEST['HTTP_USER_AGENT'], 'MSIE')>0:
          options = (options[:1]
                     + ( {'label':'Editor', 'action':'manage_IEEditorForm'}, )
                     + options[1:])


        return options


    meta_type = 'XConteudo'

    def __init__(self, id, title):
        self.id = id
        self.title = title

    def __setstate__(self, state):
      XConteudo.inheritedAttribute('__setstate__')(self, state)
      if self.getDocumentFormat() =='ZPT' and hasattr(self, '_cook'):
          self._cook()


    def PrincipiaSearchSource(self):
	  """Suporte para busca."""
	  return self.getTexto()

    security.declareProtected('View source', 'document_src')
    def document_src(self, REQUEST=None, RESPONSE=None):
	"""Retorna o source do documento."""
	if RESPONSE is not None:
	   RESPONSE.setHeader('Content-Type', 'text/plain')
	return self.getTexto()

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None, *args, **kw):
        "Retorna o conteudo do objeto como HTML."
        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        if REQUEST is None:
          REQUEST = self.REQUEST

        if self.checkPublished() or self.canEdit():
          body = self.getTexto()
          if self.getDocumentFormat()=='STX':
            r=STX(body, header=0)
          if self.getDocumentFormat()=='DTML':
            r=DTML(body, mapping=kw)(self, REQUEST)
          if self.getDocumentFormat()=='ZPT':
            r = self.pt_render(extra_context={'options': kw})
          if self.getDocumentFormat()=='HTML':
            r=body
          if self.getDocumentFormat()=='PLAIN':
            r=body
        else:
          return self.errorStatusPublicacao(REQUEST)
        if not self._cache_namespace_keys:
          self.ZCacheable_set(r)
        return r

    def __call__(self, REQUEST=None, *args, **kw):
        if REQUEST is None:
          REQUEST = self.REQUEST
        return self.render(REQUEST, *args, **kw)

    def __str__(self, REQUEST=None, *args, **kw):
        if REQUEST is None:
          REQUEST = self.REQUEST
        return self.getTexto()

    def logError(productid='XConteudo'):
        exc = sys.exc_info()
        try:
            LOG('Render', ERROR, 'Excecao enquanto renderizava: \n %s'
                % productid, error=exc)
        finally:
            exc = None

    security.declareProtected('Access contents information', 'summary')
    def summary(self, REQUEST=None):
        "Resumo do Texto Pagina"
        txtresumo = ""
        if not hasattr(self, '_summary') or (hasattr(self,'_summary') and (self._summary=="" or self._summary is None)):
          try:
            txtresumo = self.render(REQUEST=self.REQUEST)
          except:
            self.logError()
            pass
        else:
          txtresumo = self._summary

        if txtresumo is not None:
          if _no_stripogram:
            txtresumo =  txtresumo
          else:
            txtresumo = html2text(txtresumo)
          pos  = string.rfind(txtresumo, '\n', 0, 1000)
          if pos > 0:
            txtresumo = txtresumo[:pos]
          if self._summary != txtresumo:
            self._summary = txtresumo
          return self._summary
        return self.title_or_id()

    security.declareProtected('Modificar propriedades', 'manage_IEEditorProperties')
    def manage_IEEditorProperties(self, texto="", in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XNoticia"
        if self.canEdit():
          self._text = texto
          if self.getDocumentFormat() == 'ZPT' and hasattr(self, '_cook'):
            self._cook()
          self.manage_catalogIndex()
          self.ZCacheable_invalidate()

        if REQUEST is not None:
          return self.manage_IEEditorForm(REQUEST, management_view='Propriedades',
              manage_tabs_message='A Noticia <strong>%s</strong> foi editada.' % self.id
              )

    security.declareProtected('Modificar Propriedades do XConteudo','manage_editXConteudo')
    def manage_editXConteudo(self,  REQUEST=None):
        """ Modifica as Propriedades do XConteudo """

Globals.InitializeClass(XConteudo)

def manage_addXConteudo(self, REQUEST=None):
        """ Adiciona um XConteudo """

manage_addXConteudoForm = Globals.HTMLFile('dtml/addXConteudoForm', globals())
