from Globals import InitializeClass
from Globals import Persistent, Acquisition
import Globals
import AccessControl
import OFS
from OFS.Cache import Cacheable
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from AccessControl import ClassSecurityInfo
from XWorkFlow import XWorkFlow
from ZCoMIX_Common import ZCoMIX_Common
from XConteudo import XConteudo
from DateTime import DateTime

class XLink (XConteudo):
    """Classe que define o XLink."""

    friendly_name = 'Link'
    title = None
    abs_url = None
    link_target='_self'
    estilo_css = None
    default_targets = [('Nova Janela','_blank'), ('Mesma Janela','_self')]
    security = ClassSecurityInfo()
    meta_type = 'XLink'

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'link_target','type':'string','mode':'w' },
        {'id':'estilo_css','type':'string','mode':'w' },
        {'id':'ordem_layout','type':'int','mode':'w' },
        {'id':'mostrar_no_menu_superior','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_superior','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        )

    security.declareProtected('Modificar Propriedades do XLink', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXLinkForm", globals(), in_ZMI=1)

    def __init__(self, id, title=None, url=None):
        self.id = id
        self.title = title
        self.abs_url = url

    def PrincipiaSearchSource(self):
	  """Suporte para busca."""
	  return "%s (%s)" % (self.title_or_id(), self.getLinkURL())

    def getTexto(self, REQUEST=None):
      """ """
      return "%s (%s)" % (self.title_or_id(), self.getLinkURL())

    def __str__(self, REQUEST=None):
      """ """
      return self.getTexto()

    #security.declareProtected('Access contents information','absolute_url')
    #def absolute_url(self, REQUEST = None, relative = 0):
    #    if self.abs_url is not None:
    #            return self.abs_url
    #    else:
    #            return "#"

    security.declareProtected('Access contents information','getLinkURL')
    def getLinkURL(self, REQUEST = None):
        if self.abs_url is not None:
            return self.abs_url
        else:
            return "%s" % self.aq_parent.absolute_url()

    security.declareProtected('View','index_html')
    def index_html(self, REQUEST = None):
        """ """
        url = self.getLinkURL()
        if url[:4] != 'http' and url[0]!='/':
          url = "%s/%s" % (self.absolute_url(), url)
        return self.REQUEST.RESPONSE.redirect(url)

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None):
        "Retorna o conteudo do objeto como HTML."
        result = ""
        if self.estilo_css is None:
                result += "<a href='%s' target='%s'>%s</a>" % (self.absolute_url(), self.link_target, self.title_or_id())
        else:
                result += "<a class='%s' href='%s' target='%s'>%s</a>" % (self.estilo_css, self.absolute_url(), self.link_target, self.title_or_id())
        return result

    security.declareProtected('Modificar Propriedades do XLink', 'manage_Properties')
    def manage_Properties(self, title=None, url=None,
        mostrar_no_menu_superior=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0,
        summary=None, keywords=None,
        link_target='_self', estilo_css=None, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XLink"
        if self.canEdit():
          self.title = title
          self.abs_url = url
          self.link_target=link_target
          self.mostrar_no_menu_superior = mostrar_no_menu_superior
          self.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
          self.mostrar_no_menu_direito = mostrar_no_menu_direito
          self.estilo_css = estilo_css
          self._summary = summary
          self._keywords = keywords
          self.manage_catalogIndex()

        if REQUEST is not None:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='O Link <strong>%s</strong> foi editado.' % self.id
                )

InitializeClass(XLink)

def manage_addXLink(self, id, title=None, url=None,
        mostrar_no_menu_superior=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0,
        data_inicial=None, data_final=None, summary=None, keywords=None,
        link_target='_self', estilo_css=None, categorias=None, novas_categorias=None, REQUEST=None):
        """Cria um XLink e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """
        link = XLink(id, title, url)
        self._setObject(id, link)
        link = self.restrictedTraverse(id)
        link.mostrar_no_menu_superior = mostrar_no_menu_superior
        link.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        link.mostrar_no_menu_direito = mostrar_no_menu_direito
        link.link_target = link_target
        link.estilo_css = estilo_css
        link.manage_PeriodoExibicao(data_inicial = data_inicial, data_final = data_final)
        link._data_criacao = DateTime()
        if novas_categorias:
            if categorias is None:
              categorias = novas_categorias
            else:
              categorias.extend(novas_categorias)
        if categorias is None:
            categorias = []
        for i in range(0, categorias.count("")):
          try: categorias.remove("")
          except: pass
        link.categorias = categorias
        link._summary = summary
        link._keywords = keywords
        link.manage_workflow_statusProduzindo()
        link.manage_catalogIndex()
        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXLinkForm = Globals.HTMLFile('dtml/addXLinkForm', globals())
