import XConteudo, OFS, Globals, time
from DateTime import DateTime
from Globals import Persistent
from AccessControl import ClassSecurityInfo
from string import split
from OFS.Folder import Folder
from urllib import quote
from OFS.Folder import Folder
from OFS.Cache import Cacheable
from DocumentTemplate.DT_HTML import HTML as DTML
from StructuredText.StructuredText import HTML as STX
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate as ZPT
from utilities import dt2us

_marker = []

class XNoticia ( XConteudo.XConteudo ):
    """Classe que define a XNoticia."""

    meta_type = 'XNoticia'
    friendly_name = 'Notcia'
    security = ClassSecurityInfo()
    _usar_structured_text = 1

    security.declareProtected('Modificar Propriedades da XNoticia', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXNoticiaForm", globals(), in_ZMI=1)

    def manage_options(self, REQUEST=None):
      manage_options=(
          ({'label':'Propriedades', 'action':'manage_PropertiesForm'},
            {'label':'Categorias', 'action':'manage_CategoriasForm'},
            {'label':'Workflow', 'action':'manage_WorkFlowForm'},
            {'label':'Status', 'action':'manage_StatusForm'},
            )
          + Folder.manage_options[:1] +
          ({'label':'Visualizar', 'action': '' },)
          + Folder.manage_options[3:]
          + Cacheable.manage_options)

      zcomix_manage_options=((
          {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
          {'label':'Categorias', 'action':'manage_ZCategoriasForm'},
          {'label':'Workflow', 'action':'manage_ZWorkFlowForm'},
          {'label':'Status', 'action':'manage_ZStatusForm'},
          {'label':'Visualizar', 'action':''},))

      if self.SecurityGetUser().has_permission('View management screens', self):
        return manage_options
      else:
        return zcomix_manage_options

    def __init__(self, id, title):
        self.id = id
        self.title = title

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None, *args, **kw):
        """Retorna o conteudo do objeto como HTML."""
        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        if self.checkPublished() or self.canEdit():
          body = "<h1>%s</h1>\n\n" % self.title_or_id()
          body += "%s\n" % self.getTexto()
          if self.getDocumentFormat()=='STX':
            r=STX(body, header=0, level=2)
          if self.getDocumentFormat()=='DTML':
            r=DTML(body)(self, REQUEST)
          if self.getDocumentFormat()=='ZPT':
            ob = ZPT(id=self.getId(), text=body)
            ob = ob.__of__(self)
            r = ob.pt_render()
          if self.getDocumentFormat()=='HTML':
            r=body
          if self.getDocumentFormat()=='PLAIN':
            r=body
        else:
          return self.errorStatusPublicacao(REQUEST)

        if not self._cache_namespace_keys:
            self.ZCacheable_set(r)
        return r

    security.declareProtected('Modificar Propriedades da XNoticia', 'manage_Properties')
    def manage_Properties(self, title, mostrar_no_menu=0, texto="", usar_structured_text=0,
		summary=None, keywords=None, document_format='DTML', in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XNoticia"
        if self.canEdit():
          self.title = title
          self._text = texto
          self._document_format = document_format
          self._usar_structured_text = usar_structured_text
          self._summary = summary
          self._keywords = keywords
          if self.getDocumentFormat() == 'ZPT' and hasattr(self, '_cook'):
            self._cook()
          self.manage_catalogIndex()
          self.ZCacheable_invalidate()

        if REQUEST is not None:
          return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
              manage_tabs_message='A Noticia <strong>%s</strong> foi editada.' % self.id
              )

Globals.default__class_init__(XNoticia)

def manage_addXNoticia(self, title, produtor='admin',
        data_inicial=None, data_final=None, mostrar_no_menu=0, summary=None, keywords=None,
        texto="", categorias=None, novas_categorias=None, document_format='DTML', REQUEST=None):
        """Cria um XNoticia e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """

        id = str(int(DateTime()))
        noticia = XNoticia(id, title)
        noticia.produtor = REQUEST.AUTHENTICATED_USER.getId()
        if novas_categorias:
            if categorias is None:
              categorias = novas_categorias
            else:
              categorias.extend(novas_categorias)
        if categorias is None:
            categorias = []
        for i in range(0, categorias.count("")):
          try: categorias.remove("")
          except: pass
        noticia.categorias = categorias
        noticia._summary = summary
        noticia._keywords = keywords
        noticia._document_format = document_format
        noticia.manage_PeriodoExibicao(data_inicial = data_inicial, data_final = data_final)
        noticia._text = texto
        noticia._data_criacao = DateTime()
        self._setObject(id, noticia)
        noticia = self.restrictedTraverse(id)
        noticia.manage_workflow_statusProduzindo()
        if noticia.getDocumentFormat() == 'ZPT' and hasattr(noticia, '_cook'):
          noticia._cook()
        noticia.manage_catalogIndex()

        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXNoticiaForm = Globals.DTMLFile('dtml/addXNoticiaForm', globals())
