import Globals, AccessControl, OFS, Products, time
from Globals import Persistent, Acquisition
from AccessControl import ClassSecurityInfo, SecurityManagement
from AccessControl.Role import RoleManager
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from XWorkFlow import XWorkFlow
from DocumentTemplate.DT_HTML import HTML
from StructuredText.StructuredText import HTML as STXHTML
from DateTime import DateTime
from XWorkFlow import XWorkFlow
from Catalogavel import Catalogavel
from ZCoMIX_Common import ZCoMIX_Common
from Categorizavel import Categorizavel
from PropriedadesGerenciaveis import PropriedadesGerenciaveis

class XSecao ( PropriedadesGerenciaveis, Categorizavel, Catalogavel, XWorkFlow, ZCoMIX_Common, OFS.Folder.Folder ):
    """Classe que define a XSecao."""

    friendly_name = 'Seo'
    responsavel = ''
    email_responsavel = ''
    responsavel_revisao = ''
    responsavel_publicacao = ''
    monitorada = 0
    data_inicial = DateTime('2001/01/01')
    data_final = DateTime('2030/01/01')
    pagina_inicial = None
    template = None
    mostrar_no_menu_superior = 0
    mostrar_no_menu_esquerdo = 1
    mostrar_no_menu_direito = 0
    ordem_menu_superior = 0
    ordem_menu_esquerdo = 0
    ordem_menu_direito = 0
    usar_menu_esquerdo_pai = 0
    usar_menu_direito_pai = 0
    property_extensible_schema__ = 1

    # Compatibilidade com versoes anteriores
    mostrar_no_menu = mostrar_no_menu_esquerdo
    mostrar_no_menu_principal = mostrar_no_menu_superior
    ordem_menu_principal = ordem_menu_superior
    ordem_menu_secao = ordem_menu_esquerdo
    usar_menu_pai = usar_menu_esquerdo_pai

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'responsavel','type':'string','mode':'w'},
        {'id':'email_responsavel','type':'string','mode':'w'},
        {'id':'responsavel_revisao','type':'string','mode':'w'},
        {'id':'responsavel_publicacao','type':'string','mode':'w'},
        {'id':'monitorada','type':'boolean','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'pagina_inicial','type':'string','mode':'w' },
        {'id':'categorias','type':'string','mode':'w' },
        {'id':'_keywords','type':'string','mode':'w' },
        {'id':'template','type':'string','mode':'w' },
        {'id':'mostrar_no_menu_superior','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'mostrar_no_menu_direito','type':'boolean','mode':'w' },
        {'id':'ordem_menu_superior','type':'boolean','mode':'w' },
        {'id':'ordem_menu_esquerdo','type':'boolean','mode':'w' },
        {'id':'ordem_menu_direito','type':'boolean','mode':'w' },
        {'id':'usar_menu_esquerdo_pai','type':'boolean','mode':'w' },
        {'id':'usar_menu_direito_pai','type':'boolean','mode':'w' },
        )

    meta_type = 'XSecao'
    security = ClassSecurityInfo()

    security.declareProtected('View management screens','manage_main')
    manage_main = Globals.DTMLFile("dtml/main", globals(), in_ZMI=1)

    security.declareProtected('View management screens', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXSecaoForm", globals(), in_ZMI=1)

    security.declareProtected('View management screens','gadgetMenuForm')
    gadgetMenuForm = Globals.DTMLFile("dtml/editXSecaoMenuForm", globals())

    security.declareProtected('View','errorPaginaInicial')
    errorPaginaInicial = Globals.DTMLFile("dtml/errorPaginaInicial", globals())

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] not in ('XTemplate', 'XParagrafo', 'XImagem')
        return filter(f, Products.meta_types)

    def manage_options(self, REQUEST=None):
      options = ( OFS.Folder.Folder.manage_options[:1] + (
          {'label':'Propriedades', 'action':'manage_PropertiesForm', 'help':('ZCoMIX','XSecao_Propriedades.stx')},
          {'label':'Categorias', 'action':'manage_CategoriasForm'},
          {'label':'Workflow', 'action':'manage_WorkFlowForm', 'help':('ZCoMIX','XWorkflow_Workflow.stx')},
          {'label':'Visualizar', 'action': '' },)
          #+ MultiCatalogAware.manage_options
          + OFS.Folder.Folder.manage_options[3:]
          )

      return options

    def __init__(self, id, title):
        self.id = id
        self.title = title

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None):
        "Visualizacao principal (pagina inicial) da XSecao"
        url = self.absolute_url()
        pagina_inicial = self.getPaginaInicial()
        if pagina_inicial is not None:
          url += '/' + pagina_inicial
          return REQUEST.RESPONSE.redirect(url)
        else:
          return self.errorPaginaInicial(REQUEST)

    security.declarePublic('Access contents information', 'getTemplate')
    def getTemplate(self, REQUEST=None):
        "Retorna o Template, caso existir ou o Template Padrao"

        if hasattr(self, 'template') and self.template is not None:
          return self.template
        if self.aq_parent.meta_type=='XSite':
          return self.getTemplatePrincipal()
        if hasattr(self.aq_parent, 'getTemplate'):
          return self.aq_parent.getTemplate()
        return self.getTemplatePrincipal()

    security.declareProtected('Access contents information', 'getPaginaInicial')
    def getPaginaInicial(self, REQUEST=None):
        "Retorna a pagina inicial, caso existir ou o primeiro XPagina, ou a primeira XSecao"
        pgini = self.pagina_inicial
        if not pgini:
                for pg in self.objectValues(['XPagina']):
                        pgini = pg.getId()
                        break

        if not pgini:
                for pg in self.objectValues(['XSecao']):
                        pgini = pg.getId()
                        break

        if not pgini:
                pgini = 'index_html'

        return pgini

    security.declareProtected('Access contents information', 'summary')
    def summary(self, REQUEST=None):
        "Resumo do Texto Secao"
        if hasattr(self, '_summary'):
          if self._summary is None:
            txtresumo = ""
            for i in self.objectValues(['XLink','XPagina','XSecao','XNoticia']):
              txtresumo += "%s, " % i.title_or_id()
            if len(txtresumo)>2 and txtresumo[len(txtresumo)-2] == ',':
              txtresumo = txtresumo[:len(txtresumo)-2]
            return txtresumo
          return self._summary
        return ""

    security.declareProtected('Access contents information', 'menuItems')
    def menuItems(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Esquerdo da XSecao"
        return self.itensMenu(esquerdo=1, REQUEST=REQUEST)

    security.declareProtected('Access contents information', 'itensMenuEsquerdo')
    def itensMenuEsquerdo(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Esquerdo da XSecao"
        return self.itensMenu(esquerdo=1, REQUEST=REQUEST)

    security.declareProtected('Access contents information', 'itensMenuDireito')
    def itensMenuDireito(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu Direito da XSecao"
        return self.itensMenu(esquerdo=0, REQUEST=REQUEST)

    security.declareProtected('Access contents information', 'itensMenu')
    def itensMenu(self, esquerdo=1, REQUEST=None):
        "Lista de items a serem mostrados no Menu da XSecao"
        items = []
        secao = self
        if esquerdo:
          if hasattr(self, 'usar_menu_esquerdo_pai') and self.usar_menu_esquerdo_pai:
            if self.aq_inner.aq_parent.meta_type=='XSite':
              secao = self.restrictedTraverse(self.getSecaoPrincipal())
            else:
              secao = self.aq_inner.aq_parent
              return secao.itensMenu(esquerdo=esquerdo, REQUEST=REQUEST)
          for item in secao.objectValues(['XPagina', 'XPaginaLayout', 'XSecao', 'XLink']):
              if item.mostrar_no_menu_esquerdo and item.checkPublished():
                items.append(item)
        else:
          if hasattr(self, 'usar_menu_direito_pai') and self.usar_menu_direito_pai:
            if self.aq_inner.aq_parent.meta_type=='XSite':
              secao = self.restrictedTraverse(self.getSecaoPrincipal())
            else:
              secao = self.aq_inner.aq_parent
              return secao.itensMenu(esquerdo=esquerdo, REQUEST=REQUEST)
          for item in secao.objectValues(['XPagina', 'XPaginaLayout', 'XSecao', 'XLink']):
              if item.mostrar_no_menu_direito and item.checkPublished():
                items.append(item)

        if len(items)>1:
          if esquerdo:
            if items[0].ordem_menu_esquerdo == items[1].ordem_menu_esquerdo:
              items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
            else:
              items.sort(lambda x, y: cmp(x.ordem_menu_esquerdo, y.ordem_menu_esquerdo))
          else:
            if items[0].ordem_menu_direito == items[1].ordem_menu_direito:
              items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
            else:
              items.sort(lambda x, y: cmp(x.ordem_menu_direito, y.ordem_menu_direito))
        return items

    security.declareProtected('Modificar propriedades','manage_changeUsarPai')
    def manage_changeUsarPai(self, usar_menu_pai=0, esquerdo=0, in_ZMI=1, REQUEST=None, *args, **kw):
        "Utilizar menu da secao Pai"
        if self.canEdit():
          if esquerdo:
                  self.usar_menu_esquerdo_pai = usar_menu_pai
          else:
                  self.usar_menu_direito_pai = usar_menu_pai

        if REQUEST is not None:
          return self.manage_PropertiesForm(REQUEST, management_view = 'Propriedades',
                        manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)

    security.declareProtected('Modificar propriedades','manage_Menu')
    def manage_Menu(self, menu_items=[], esquerdo=0, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica a ordem dos Menus da XSecao"
        if self.canEdit():
          ordem = 0
          for i in menu_items:
                  ob = getattr(self.aq_base, i, None)
                  if ob is not None:
                          if esquerdo:
                                  ob.ordem_menu_esquerdo = ordem
                          else:
                                  ob.ordem_menu_direito = ordem
                  ordem += 1

        if REQUEST is not None:
          return self.manage_PropertiesForm(REQUEST, management_view = 'Propriedades',
                  manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id)

    security.declareProtected('Modificar propriedades', 'manage_Properties')
    def manage_Properties(self, title, responsavel='admin', email_responsavel='', \
        responsavel_revisao='', responsavel_publicacao='', monitorada=0, \
        pagina_inicial=None, \
        mostrar_no_menu_esquerdo=0, mostrar_no_menu_direito=0, mostrar_no_menu_superior=0, \
        summary=None, keywords=None, template=None, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades da XSecao"
        if self.canEdit():
          self.title = title
          self.responsavel = responsavel
          self.email_responsavel = email_responsavel
          self.responsavel_revisao = responsavel_revisao
          self.responsavel_publicacao = responsavel_publicacao
          self.monitorada = monitorada
          self.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
          self.mostrar_no_menu_direito = mostrar_no_menu_direito
          self.mostrar_no_menu_superior = mostrar_no_menu_superior
          if not pagina_inicial:
                  for i in self.objectValues('XPagina'):
                          pagina_inicial = i.getId()
          self.pagina_inicial = pagina_inicial
          self.template = template
          self._summary = summary
          if keywords is not None:
            for i in keywords:
              if i =="":
                keywords.remove("")
          self._keywords = keywords
          self.manage_catalogIndex()

        if REQUEST is not None:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Secao <strong>%s</strong> foi editada.' % self.id
                )

Globals.InitializeClass(XSecao)

def manage_addXSecao(self, id, title, responsavel='admin', email_responsavel='', \
        responsavel_revisao='', responsavel_publicacao='', monitorada=0, \
        data_inicial=None, data_final=None, pagina_inicial=None, template=None, \
        mostrar_no_menu_direito=0, mostrar_no_menu_esquerdo=0, mostrar_no_menu_superior=0, \
        summary=None, keywords=None, REQUEST=None):
        """Cria um XSecao e o adiciona ao seu parent Folder.
        O argumento 'self' referencia o parent Folder.
        """
        secao = XSecao(id, title)
        secao.responsavel = responsavel
        secao.email_responsavel = email_responsavel
        secao.responsavel_revisao = responsavel_revisao
        secao.responsavel_publicacao = responsavel_publicacao
        secao.monitorada = monitorada
        secao.manage_PeriodoExibicao(data_inicial = data_inicial, data_final = data_final)
        secao._data_criacao = DateTime()
        secao.pagina_inicial = pagina_inicial
        secao.template = template
        secao._summary = summary
        if keywords is not None:
          for i in keywords:
            if i =="":
              keywords.remove("")
        secao._keywords = keywords
        secao.mostrar_no_menu_esquerdo = mostrar_no_menu_esquerdo
        secao.mostrar_no_menu_direito = mostrar_no_menu_direito
        secao.mostrar_no_menu_superior = mostrar_no_menu_superior
        self._setObject(id, secao)
        secao = self.restrictedTraverse(id)
        secao.manage_workflow_statusProduzindo()
        secao.manage_catalogIndex()
        if REQUEST is not None:
                return self.manage_main(self, REQUEST)

manage_addXSecaoForm = Globals.HTMLFile('dtml/addXSecaoForm', globals())
