import Globals, OFS, AccessControl, Products, time, os, sys
from Globals import Persistent, Acquisition
from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
from OFS.Application import Application
from App.Management import Tabs
from AccessControl import SecurityManagement
from AccessControl import getSecurityManager, Unauthorized
from XWorkFlow import XWorkFlow
from ZPublisher.BeforeTraverse import registerBeforeTraverse, NameCaller
from zLOG import LOG, ERROR, WARNING, INFO
from ZCoMIX_Common import ZCoMIX_Common
from DateTime import DateTime
from OFS.ObjectManager import REPLACEABLE
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from PropriedadesGerenciaveis import PropriedadesGerenciaveis

class XSite ( PropriedadesGerenciaveis, ZCoMIX_Common, XWorkFlow, OFS.Folder.Folder ):
    """Classe que define o XSite."""

    __ac_roles__ = ('Produtor','Revisor','Publicador')

    friendly_name = 'Site'
    responsavel = ""
    data_inicial = DateTime("1970/01/01")
    data_final = DateTime("2030/01/01")
    template_principal = None
    secao_principal = None
    catalog_principal = None
    email_responsavel_site = "webmaster@localhost"
    categorias = None
    _catalog_workflow = None

    security = ClassSecurityInfo()

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] not in ('XPagina', 'XNoticia', 'XParagrafo','XImagem')
        return filter(f, OFS.Folder.Folder.all_meta_types(self))

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'responsavel','type':'string','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'template_principal','type':'string','mode':'w' },
        {'id':'secao_principal','type':'string','mode':'w' },
        {'id':'catalog_principal','type':'string','mode':'w' },
        )

    security.declareProtected('Access contents information', 'manage_options')
    def manage_options(self, REQUEST=None):
        options=(OFS.Folder.Folder.manage_options[:1] +(
            {'label':'Propriedades', 'action':'manage_PropertiesForm', 'help':('ZCoMIX', 'XSite_Propriedades.stx')},
            {'label':'Menu', 'action':'manage_MenuForm', 'help':('ZCoMIX', 'XSite_Menu.stx')},
            {'label':'Visualizar', 'action':''},
            {'label':'Noticias da Capa', 'action':'manage_SiteNewsForm'},
            ) + OFS.Folder.Folder.manage_options[3:] )


        if self.getCatalogWorkFlow() is not None:
          options = (options[:2] +
          ({'label':'Workflow', 'action':self.getCatalogWorkFlow()+'/manage_workspace'},)
          +options[2:])

        return options

    meta_type = 'XSite'

    security.declareProtected('View management screens','manage_main')
    manage_main = Globals.DTMLFile("dtml/main", globals(), in_ZMI=1)

    security.declareProtected('View management screens','manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXSiteForm", globals(), in_ZMI=1)

    security.declareProtected('View management screens','manage_MenuForm')
    manage_MenuForm = Globals.DTMLFile("dtml/editXSiteMenuForm", globals(), in_ZMI=1)

    security.declareProtected('View management screens','manage_SiteNewsForm')
    manage_SiteNewsForm = Globals.DTMLFile("dtml/editNoticiasCapaForm", globals(), in_ZMI=1)

    security.declareProtected('View management screens','gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXSecaoForm", globals(), in_ZMI=0)

    security.declareProtected('View management screens','gadgetMenuForm')
    gadgetMenuForm = Globals.DTMLFile("dtml/editXSecaoMenuForm", globals())

    security.declareProtected('View','errorSecaoPrincipal')
    errorSecaoPrincipal = Globals.DTMLFile("dtml/errorSecaoPrincipal", globals())

    security.declareProtected('View management screens','editor_browse_files')
    editor_browse_files = Globals.DTMLFile("dtml/editor_browse_files", globals())

    security.declareProtected('View management screens','editor_browse_images')
    editor_browse_images = Globals.DTMLFile("dtml/editor_browse_images", globals())

    def __init__(self, id, title):
        self.id = id
        self.title = title

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None):
        "Visualizacao principal (pagina inicial) do XSite"
        secao_principal = self.getSecaoPrincipal()
        if secao_principal is not None:
          url = self.absolute_url()
          url += '/' + secao_principal
          return REQUEST.RESPONSE.redirect(url)
        else:
          return self.errorSecaoPrincipal(REQUEST)

    index_html.__replaceable__ = REPLACEABLE

    def SecurityGetUser(self):
      return (SecurityManagement.getSecurityManager().getUser())

    security.declareProtected('Access contents information','getXSitePath')
    def getXSitePath(self, REQUEST=None):
        """Retorna o path do XSite"""
        return '/%s' % self.absolute_url(relative=1)

    security.declareProtected('Access contents information','canEdit')
    def canEdit(self, REQUEST=None):
        """ """
        return 1

    security.declareProtected('Access contents information','utilizarIEEditor')
    def utilizarIEEditor(self, REQUEST=None):
        """ """
        if hasattr(self, '_utilizar_ieeditor'):
          return self._utilizar_ieeditor
        else:
          if hasattr(self, 'IEEditor'):
            self._utilizar_ieeditor = 1
            return 1

    security.declareProtected('Access contents information','menuItemsPrincipal')
    def menuItemsPrincipal(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu do XSite"
        return self.itensMenuSuperior(REQUEST=REQUEST)

    security.declareProtected('Access contents information','getNoticiasCapa')
    def getNoticiasCapa(self, REQUEST=None):
        """ Lista de Noticias selecionadas para Capa do Site """
        result = []
        if hasattr(self,'_noticias_capa'):
          for i in self._noticias_capa:
            try:
              i = eval(i)
            except:
              pass
            noticia = self.restrictedTraverse(i, None)
            if noticia is not None:
              result.append(noticia)

        return result

    security.declareProtected('Access contents information','itensMenuSuperior')
    def itensMenuSuperior(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu do XSite"

        if REQUEST is not None:
                for i in range(0, len(self.REQUEST.PARENTS)):
                        dir = self.REQUEST.PARENTS[i]
                        if dir.meta_type=='XSite':
                                site = dir
                                break
        else:
                site = self

        items = []
        for item in site.objectValues(['XSecao', 'XPagina','XLink']):
                if self.SecurityGetUser().has_permission('Access contents information', item) and item.mostrar_no_menu_superior and item.checkPublished():
                        items.append(item)

        if len(items)>1:
                if items[0].ordem_menu_superior == items[1].ordem_menu_superior:
                        items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
                else:
                        items.sort(lambda x, y: cmp(x.ordem_menu_superior, y.ordem_menu_superior))

        return items

    def __before_publishing_traverse__(self, request=None, *args, **kw):
        if hasattr(request, 'TraversalRequestNameStack'):
          stack = request['TraversalRequestNameStack']
        else:
          return

        method=request.get('REQUEST_METHOD', 'GET')
        if not method in ('GET', 'POST'):
            return

        if not stack:
          return

        try:
          if stack[0][:6]=='manage' or stack[-1][:6]=='manage':
            return
        except:
          pass

        obj = None
        template = None

        path = request.path[:]
        path.reverse()
        try:
          obj = self.unrestrictedTraverse(path)
        except:
          pass

        try:
          if not hasattr(obj, 'getTemplate'):
            if len(path) > 1:
              root = getattr(self, path[-2:-1], None)
              if root is not None and root.meta_type is not 'Virtual Host Monster' \
              and root.meta_type is not 'Site Root':
                for i in range(1, len(path)):
                  obj = self.unrestrictedTraverse(path[:-i])
                  if hasattr(obj, 'getTemplate'):
                    break
        except:
          pass

        try:
          template = obj.getTemplate()
        except:
          pass

        if template is None:
          try:
            template = obj.getTemplatePrincipal()
          except:
            pass

        if template is None:
          return

        if (not template in stack) and (not template in path):
          request['TraversalRequestNameStack'].append(template)

    security.declareProtected('Access contents information','getTemplatePrincipal')
    def getTemplatePrincipal(self, REQUEST=None):
        if self.template_principal is None:
          template_principal = self.objectValues("XTemplate")
          try:
            template_principal = template_principal[0].getId()
            self.template_principal = template_principal
          except:
            template_principal = None
        else:
          template_principal = self.template_principal
        return template_principal

    getTemplate = getTemplatePrincipal

    security.declareProtected('Access contents information','getCatalogPrincipal')
    def getCatalogPrincipal(self, REQUEST=None):
        if self.catalog_principal is None:
          catalog_principal = self.objectValues("ZCatalog")
          try:
                catalog_principal = catalog_principal[0].getId()
                self.catalog_principal = catalog_principal
          except:
                catalog_principal = None
        else:
          catalog_principal = self.catalog_principal
        return catalog_principal

    security.declareProtected('Access contents information','getCatalogWorkFlow')
    def getCatalogWorkFlow(self, REQUEST=None):
        if self._catalog_workflow is None:
          catalog_workflow = self.objectValues("XWorkFlowCatalog")
          try:
                catalog_workflow = catalog_workflow[0].getId()
                self._catalog_workflow = catalog_workflow
          except:
                catalog_workflow = None
        else:
          catalog_workflow = self._catalog_workflow
        return catalog_workflow

    security.declareProtected('Access contents information','getSecaoPrincipal')
    def getSecaoPrincipal(self, REQUEST=None):
        if self.secao_principal is None:
          secao_principal = self.objectValues("XSecao")
          try:
                secao_principal = secao_principal[0].getId()
                self.secao_principal = secao_principal
          except:
                secao_principal = None
        else:
          secao_principal = self.secao_principal
        return secao_principal

    security.declareProtected('Gerenciar Noticias da Capa','manage_siteNews')
    def manage_siteNews(self, noticias_capa=None, in_ZMI=1, REQUEST=None, *args, **kw):
        """ Modifica a selecao de noticias da capa do site  """
        if noticias_capa is None:
            noticias_capa = []
        self._noticias_capa = noticias_capa

        if REQUEST is not None:
            return self.manage_SiteNewsForm(REQUEST, management_view = 'Noticias da Capa',
                    manage_tabs_message='As noticias da capa do site <strong>%s</strong> foram modificadas.' % self.id)

    security.declareProtected('Gerenciar Noticias da Capa','manage_siteNews')
    def manage_catSiteNews(self, categorias=None, novas_categorias=None, in_ZMI=1, REQUEST=None, *args, **kw):
        """ Modifica a selecao de noticias da capa do site  """

        if novas_categorias:
            if categorias is None:
              categorias = novas_categorias
            else:
              categorias.extend(novas_categorias)
        if categorias is None:
            categorias = []
        for i in range(0, categorias.count("")):
          try: categorias.remove("")
          except: pass
        self.categorias = categorias

        if REQUEST is not None:
            return self.manage_SiteNewsForm(REQUEST, management_view = 'Noticias da Capa',
                    manage_tabs_message='As categorias das noticias da capa do site <strong>%s</strong> foram modificadas.' % self.id)


    security.declareProtected('Modificar propriedades','manage_Menu')
    def manage_Menu(self, menu_items, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica a ordem dos Menus do XSite"
        ordem = 0
        for i in menu_items:
                ob = getattr(self, i, None)
                if ob is not None:
                        ob.ordem_menu_superior = ordem
                ordem += 1

        if REQUEST is not None:
            return self.manage_MenuForm(REQUEST, management_view = 'Menu',
                    manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)

    security.declareProtected('Modificar propriedades','manage_Properties')
    def manage_Properties(self, title, responsavel='admin', template_principal=None, secao_principal=None, catalog_principal=None, email_responsavel_site = 'webmaster@localhost', catalog_workflow=None, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades do XSite"
        self.title = title
        self.responsavel = responsavel
        self.email_responsavel_site = email_responsavel_site
        if catalog_principal is None:
          catalog_principal = self.getCatalogPrincipal()
        self.catalog_principal = catalog_principal
        if catalog_workflow is None:
          catalog_workflow = self.getCatalogWorkFlow()
        self._catalog_workflow = catalog_workflow
        if template_principal is None:
          template_principal = self.getTemplatePrincipal()
        self.template_principal = template_principal
        if secao_principal is None:
          secao_principal = self.getSecaoPrincipal()
        self.secao_principal = secao_principal
        if REQUEST is not None:
            return self.manage_PropertiesForm(REQUEST, management_view = 'Propriedades',
                    manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)

Globals.InitializeClass(XSite)


def manage_addXSite(self, id, title, responsavel='admin',
    template_principal=None, secao_principal=None, catalog_principal=None,
    email_responsavel_site = 'webmaster@localhost', catalog_workflow = None,
    criar_template_padrao=0, criar_indice_busca=0, criar_catalogo_workflow=1, utilizar_ieeditor=0, REQUEST=None):
    """Cria um XSite e o adiciona ao seu parent Folder.
    O argumento 'self' referencia o parent Folder.
    """
    site = XSite(id, title)
    self._setObject(id, site)
    site = getattr(self, id)
    site.responsavel = responsavel
    site.email_responsavel_site = email_responsavel_site
    site._utilizar_ieeditor= utilizar_ieeditor
    site._data_criacao = DateTime()

    if catalog_principal is not None:
      site.catalog_principal = catalog_principal
    else:
      if not criar_indice_busca:
        site.catalog_principal = None
      else:
        site.catalog_principal = 'catalogo'
        site.manage_addProduct['ZCatalog'].manage_addZCatalog(id='catalogo', title='Catlogo Principal')
        catalogo = getattr(site, 'catalogo')
        catalogo.manage_addColumn(name='title_or_id')
        catalogo.manage_addColumn(name='data_producao')
        catalogo.manage_addColumn(name='getStatus')
        catalogo.manage_addColumn(name='getWorkFlowStatus')
        catalogo.manage_addColumn(name='long_title')
        catalogo.manage_addColumn(name='categorias')
        catalogo.manage_addColumn(name='keywords')
        catalogo.manage_addColumn(name='absolute_url')
        catalogo.manage_addIndex(name='getStatus', type='KeywordIndex')
        catalogo.manage_addIndex(name='getWorkFlowStatus', type='KeywordIndex')
        catalogo.manage_addIndex(name='categorias', type='KeywordIndex')
        catalogo.manage_addIndex(name='keywords', type='KeywordIndex')
        catalogo.manage_addIndex(name='summary', type='TextIndex')

        busca_path = os.path.join(Globals.SOFTWARE_HOME, \
                'Products', 'ZCoMIX','add-ons','busca.zexp')
        if os.path.isfile(busca_path):
            site._importObjectFromFile(busca_path, verify=0)
        else:
            LOG('ZCoMIX Default Object Creation', INFO,
                '%s Busca import file could not be found.' % busca_path)

    if catalog_workflow is not None:
      site._catalog_workflow = catalog_workflow
    else:
      if not criar_catalogo_workflow:
        self._catalog_workflow = None
      else:
        site._catalog_workflow = 'Workflow'
        site.manage_addProduct['ZCoMIX'].manage_addXWorkFlowCatalog(id='Workflow', title='Workflow')

    if template_principal is not None:
      site.template_principal = template_principal
    else:
      if not criar_template_padrao:
        site.template_principal = None
        site.secao_principal = None
      else:
        site.template_principal = 'tplMain'
        template_path = os.path.join(Globals.SOFTWARE_HOME, \
                'Products', 'ZCoMIX','add-ons','tplMain.zexp')
        if os.path.isfile(template_path):
            site._importObjectFromFile(template_path, verify=0)
        else:
            LOG('Zope Default Object Creation', INFO,
                '%s Default Template import file could not be found.' % template_path)

        if secao_principal is not None:
          site.secao_principal = secao_principal
        else:
          site.secao_principal = 'home'
          site.manage_addProduct['ZCoMIX'].manage_addXSecao(id='home', title='Home', pagina_inicial='sobre', responsavel=responsavel, email_responsavel=email_responsavel_site, mostrar_no_menu_superior=1)
          secao = getattr(site, 'home')
          secao.manage_workflow_statusPublicado()
          about = """
                      <h2>Sobre<h2>

                        <p>Esta  a pagina inicial do seu Site. Foram criados tambm para voc um Template Padro e
                        um Catlogo Principal. Voc pode agora modificar o site conforme sua necessidade.</p>

                        <p>Mos  obra e divirta-se!</p>

                        <p><b>X3ng Web Technology</b></p>
                        """
          secao.manage_addProduct['ZCoMIX'].manage_addXPagina(id='sobre', title='Sobre', texto=about, mostrar_no_menu_esquerdo=1)
          pagina = getattr(secao, 'sobre')
          pagina.manage_workflow_statusPublicado()

    if utilizar_ieeditor:
      ieeditor_path = os.path.join(Globals.SOFTWARE_HOME, \
              'Products', 'ZCoMIX','add-ons','ieeditor.zexp')
      if os.path.isfile(ieeditor_path):
          site._importObjectFromFile(ieeditor_path, verify=0)
      else:
          LOG('ZCoMIX Default Object Creation', INFO,
              '%s IEEditor import file could not be found.' % ieeditor_path)

    registerBeforeTraverse(site, NameCaller('lambda: return'), 'Access Rule')
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

manage_addXSiteForm = Globals.HTMLFile('dtml/addXSiteForm', globals())

file = Globals.DTMLFile('dtml/manage_page_style_css', globals())
setattr(ZCoMIX_Common, 'manage_page_style.css', file)
setattr(ZCoMIX_Common, 'manage_page_style.css__roles__', None)
