import Globals, ExtensionClass, time
from Globals import InitializeClass, Persistent
from AccessControl import ClassSecurityInfo
from AccessControl import SecurityManagement
from AccessControl.Role import RoleManager
from DateTime import DateTime
from DocumentTemplate.DT_Var import url_quote_plus
from OFS import Application
from utilities import dt2us

class XWorkFlow ( ExtensionClass.Base ):
    """Classe que define o Workflow."""

    meta_type = 'XWorkFlow'
    produtor = ""
    data_producao = DateTime('2001/01/01')
    revisor = ""
    data_revisao = DateTime('2001/01/01')
    publicador = ""
    data_publicacao = DateTime('2001/01/01')

    responsavel = ""
    data_inicial = DateTime('2001/01/01')
    data_final = DateTime('2030/01/01')
    _valores_workflow_status = {}
    _valores_workflow_status['produzindo'] = 'Produzindo'
    _valores_workflow_status['produzido'] = 'Produzido'
    _valores_workflow_status['revisar'] = 'Revisar'
    _valores_workflow_status['revisando'] = 'Revisando'
    _valores_workflow_status['revisado'] = 'Revisado'
    _valores_workflow_status['rejeitado'] = 'Rejeitado'
    _valores_workflow_status['publicado'] = 'Publicado'
    _valores_workflow_status['aguardando'] = 'Aguardando'
    _valores_workflow_status['expirado'] = 'Expirado'    
    _valores_status = {}
    _valores_status['publico'] = 'Pblico'
    _valores_status['privado'] = 'Privado'
    _catalogar = 1
    _status = 'publico'
    _workflow_status = 'publicado'

    security = ClassSecurityInfo()

    security.declareProtected('Gerenciar Workflow', 'manage_WorkFlowForm')
    manage_WorkFlowForm = Globals.DTMLFile("dtml/editWorkFlowForm", globals(), in_ZMI=1)

    security.declareProtected('Access contents information', 'getWorkFlowStatus')
    def getWorkFlowStatus(self, REQUEST=None):
      """ Retorna o status do conteudo """
      status = 'publicado'
      if hasattr(self, '_workflow_status'):
        return self._workflow_status
      else:
        return 'publicado'

    security.declareProtected('Access contents information', 'getWorkFlowStatusIcon')
    def getWorkFlowStatusIcon(self, REQUEST=None):
      """ Retorna o icone do status do conteudo """
      status = self.getWorkFlowStatus()
      if status == 'publicado':
          if self.checkExpiration() < 0:
              status = 'aguardando'
          if self.checkExpiration() > 0:
              status = 'expirado'
      return 'misc_/ZCoMIX/'+ status

    security.declareProtected('Access contents information', 'getStatusIcon')
    def getStatusIcon(self, REQUEST=None):
      """ Retorna o icone do status do conteudo """
      return 'misc_/ZCoMIX/'+self.getStatus()

    security.declareProtected('Access contents information', 'getWorkFlowStatusName')
    def getWorkFlowStatusName(self, REQUEST=None):
      """ Retorna o titulo do status do conteudo """
      return self._valores_workflow_status.get(self.getWorkFlowStatus(), 'Publicado')


    security.declareProtected('Access contents information', 'getStatus')
    def getStatus(self, REQUEST=None):
      """ Retorna o status do conteudo """
      if hasattr(self, '_status'):
          return self._status
      else:
          return 'publico'

    security.declareProtected('Access contents information', 'Publico')
    def Publico(self, REQUEST=None):
      """ Retorna o status do conteudo """
      if not self.getStatus()=='privado':
        return 1
      return 0

    security.declareProtected('Access contents information', 'getStatusName')
    def getStatusName(self, REQUEST=None):
      """ Retorna o titulo do status do conteudo """
      return self._valores_status.get(self.getStatus(), 'Pblico')

    security.declareProtected('Visualizar Historico', 'getHistorico')
    def getHistorico(self, REQUEST=None):
      """ Retorna a lista do Historico """
      return self._historico

    security.declareProtected('Access contents information', 'Catalogar')
    def Catalogar(self, REQUEST=None):
      """ Catalogar ou nao o objeto. """
      return self._catalogar

    security.declareProtected('Access contents information', 'getCatalogar')
    def getCatalogar(self, REQUEST=None):
      """ Retorna o status do conteudo quanto ao catalogo """
      if self.Catalogar():
        return 'Catalogar'
      else:
        return 'No Catalogar'

    security.declareProtected('Gerenciar Status', 'manage_Catalogar')
    def manage_Catalogar(self, in_ZMI=1, REQUEST=None):
      """ Catalogar ou nao o objeto. """
      self._catalogar = not self._catalogar
      if self.Catalogar():
        self.manage_catalogIndex()
      else:
        self.manage_catalogUnIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getCatalogar() )
            )

    security.declareProtected('Gerenciar Workflow', 'manage_PeriodoExibicao')
    def manage_PeriodoExibicao(self, data_inicial=None, data_final=None, in_ZMI=1, REQUEST=None):
        """ Gerencia o periodo no qual o objeto eh visivel para o publico. """
        self.data_inicial = DateTime(dt2us(data_inicial))
        if data_final is None:
            data_final = '01/01/2030'
        self.data_final = DateTime(dt2us(data_final))

        if REQUEST is not None:
          return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
              manage_tabs_message='Novo periodo de exibicao de <strong>%s</strong>: %s - %s.' % (self.id, self.data_inicial.strftime('%d/%m/%Y %H:%M:%S'), self.data_final.strftime('%d/%m/%Y %H:%M:%S') )
              )

    security.declareProtected('Gerenciar Workflow', 'manage_addHistorico')
    def manage_addHistorico(self, historico=None, in_ZMI=1, REQUEST=None):
      """ Adiciona uma entrada ao historico """
      if not hasattr(self, '_historico'):
        self._historico = []
      if not historico is None:
        self._historico.append(historico)

    security.declareProtected('Gerenciar Status', 'manage_Status')
    def manage_Status(self, in_ZMI=1, REQUEST=None):
      """ Publicar ou nao o objeto. """
      if self.Publico():
        self._status = 'privado'
      else:
        self._status = 'publico'

      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Status',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getStatus())
            )

    security.declareProtected('Gerenciar Workflow', 'manage_workflow_statusTabs')
    def manage_workflow_statusTabs(self, REQUEST=None):
      """ Acoes disponiveis em relacao ao status """
      result = []
      if (self.getWorkFlowStatus() == 'produzido' \
          or self.getWorkFlowStatus() == 'revisar' \
          or self.getWorkFlowStatus() == 'rejeitado' \
          or self.getWorkFlowStatus() == 'revisado') \
          and self.checaPermissao('Produzir Conteudo'):
        result.append( {'id':'produzido', 'action':'manage_workflow_statusProduzindo', 'label':'Reiniciar Producao'})
      if self.getWorkFlowStatus() in ['publicado', 'aguardando', 'expirado'] \
        and self.checaPermissao('Produzir Conteudo'):
        result.append( {'id':'produzindo', 'action':'manage_workflow_statusProduzindo', 'label':'Iniciar Producao'})
        result.append( {'id':'produzindo', 'action':'manage_workflow_statusProduzindoCopia', 'label':'Iniciar Producao (com Copia)'})
      if self.getWorkFlowStatus() == 'produzindo' \
        and self.checaPermissao('Produzir Conteudo'):
        result.append( {'id':'produzido', 'action':'manage_workflow_statusProduzido', 'label':'Finalizar Producao'})
        result.append( {'id':'revisar', 'action':'manage_workflow_statusRevisar', 'label':'Enviar para Revisao'})
      if self.getWorkFlowStatus() == 'revisar' \
        and self.checaPermissao('Revisar Conteudo'):
        result.append( {'id':'revisando', 'action':'manage_workflow_statusRevisando', 'label':'Revisar Documento'})
      if self.getWorkFlowStatus() == 'revisando' \
        and self.checaPermissao('Revisar Conteudo'):
        result.append( {'id':'revisado', 'action':'manage_workflow_statusRevisado', 'label':'Aprovar Documento'})
        result.append( {'id':'rejeitado', 'action':'manage_workflow_statusRejeitado', 'label':'Rejeitar Documento'})
      if (self.getWorkFlowStatus() == 'revisado' \
          or self.getWorkFlowStatus() == 'produzido' \
          or self.getWorkFlowStatus() == 'revisar' \
          or self.getWorkFlowStatus() == 'revisando'  \
          or self.getWorkFlowStatus() == 'produzindo') \
          and self.checaPermissao('Publicar Conteudo'):
        result.append( {'id':'publicado', 'action':'manage_workflow_statusPublicado', 'label':'Publicar'})
      return result

    security.declareProtected('Produzir Conteudo', 'manage_workflow_statusProduzindo')
    def manage_workflow_statusProduzindo(self, historico="", id_original=None, in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Produzindo """
      self._workflow_status = 'produzindo'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.produtor = sm.getUser().getId()
      t = time.time()
      self.data_producao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Produzir Conteudo', 'manage_workflow_statusProduzindoCopia')
    def manage_workflow_statusProduzindoCopia(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Produzindo """
      id_original = self.getId()
      id_novo = id_original + '_' +  str(int(time.time()))
      self.aq_parent.manage_clone(self, id_novo)
      ob = getattr(self.aq_parent, id_novo)
      ob._id_original = id_original
      ob._workflow_status = 'produzindo'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':ob.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      ob.manage_addHistorico( historico=historico )
      ob.produtor = sm.getUser().getId()
      t = time.time()
      ob.data_producao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return REQUEST.RESPONSE.redirect(ob.absolute_url()+'/manage_WorkFlowForm?management_view=Workflow&manage_tabs_message='+url_quote_plus('Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())))

    security.declareProtected('Produzir Conteudo', 'manage_workflow_statusProduzido')
    def manage_workflow_statusProduzido(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Produzido """
      self._workflow_status = 'produzido'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.produtor = sm.getUser().getId()
      t = time.time()
      self.data_producao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Produzir Conteudo', 'manage_workflow_statusRevisar')
    def manage_workflow_statusRevisar(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Revisar """
      self._workflow_status = 'revisar'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.produtor = sm.getUser().getId()
      t = time.time()
      self.data_producao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Revisar Conteudo', 'manage_workflow_statusRevisando')
    def manage_workflow_statusRevisando(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Revisando """
      self._workflow_status = 'revisando'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.revisor = sm.getUser().getId()
      t = time.time()
      self.data_revisao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Revisar Conteudo', 'manage_workflow_statusRevisado')
    def manage_workflow_statusRevisado(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Revisado """
      self._workflow_status = 'revisado'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.revisor = sm.getUser().getId()
      t = time.time()
      self.data_revisao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Revisar Conteudo', 'manage_workflow_statusRejeitado')
    def manage_workflow_statusRejeitado(self, historico="", in_ZMI=1, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Rejeitado """
      self._workflow_status = 'rejeitado'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.revisor = sm.getUser().getId()
      t = time.time()
      self.data_revisao = DateTime()
      self.manage_catalogIndex()
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())
            )

    security.declareProtected('Publicar Conteudo', 'manage_workflow_statusPublicado')
    def manage_workflow_statusPublicado(self, historico="", in_ZMI=1,REQUEST=None):
      """ Modifica o status do conteudo para o estado de Publicado """
      self._workflow_status = 'publicado'
      sm = SecurityManagement.getSecurityManager()
      historico = {'data':DateTime(), 'status':self.getWorkFlowStatus(), 'usuario':sm.getUser().getId(), 'historico':historico }
      self.manage_addHistorico( historico=historico )
      self.publicador = sm.getUser().getId()
      t = time.time()
      self.data_publicacao = DateTime()
      if hasattr(self, '_id_original'):
        self.aq_parent.manage_delObjects(ids=self._id_original)
        self.aq_parent.manage_renameObject(id=self.getId(), new_id=self._id_original)
        del self._id_original
      self.manage_catalogIndex()
      if REQUEST is not None:
        return REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_WorkFlowForm?management_view=Workflow&manage_tabs_message='+url_quote_plus('Novo Status de <strong>%s</strong>: %s.' % (self.id, self.getWorkFlowStatus())))

    security.declareProtected('Access contents information', 'checkPublished')
    def checkPublished(self, REQUEST=None, *args, **kw):
      """ Checa pela data inicial e final e o status de publicado """
      if hasattr(self, 'data_inicial') and hasattr(self, 'data_final'):
        sm = SecurityManagement.getSecurityManager()
        if sm.getUser().getId() is None \
        and self.checkExpiration() == 0 \
        and self.getWorkFlowStatus()=='publicado' and self.Publico():
          return 1
        if not sm.getUser().getId() is None \
        and self.checkExpiration() == 0 \
        and self.Publico():
          return 1
      return 0

    security.declareProtected('Access contents information', 'checkExpiration')
    def checkExpiration(self, REQUEST=None, *args, **kw):
      """ Checa pela data inicial e final e o status de publicado """
      if hasattr(self, 'data_inicial') and hasattr(self, 'data_final'):
        if self.data_inicial.isFuture():
            return -1
        if self.data_inicial.isPast() \
        and self.data_final.isFuture():
            return 0
        if self.data_final.isPast():
            return 1
      return None

    security.declareProtected('Access contents information', 'canEdit')
    def canEdit(self, REQUEST=None, *args, **kw):
        """ Checa se o usuario atual tem permissao de editar o conteudo. """
        if (self.getWorkFlowStatus()=='publicado'):
          return 0
        if ((self.getWorkFlowStatus()=='produzindo')
            and self.checaPermissao('Produzir Conteudo')):
          return 1
        if ((self.getWorkFlowStatus()=='revisando')
            and self.checaPermissao('Revisar Conteudo')):
          return 1
        if ((self.getWorkFlowStatus()=='revisado')
            and self.checaPermissao('Publicar Conteudo')):
          return 1
        return 0

    security.declareProtected('Delegar Producao', 'manage_setProdutor')
    def manage_setProdutor(self, produtor="", in_ZMI=1, REQUEST=None):
      """ Delega a tarefa de producao para produtor """
      self.produtor = produtor
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Produtor de <strong>%s</strong>: %s.' % (self.id, self.produtor)
            )

    security.declareProtected('Delegar Revisao', 'manage_setRevisor')
    def manage_setRevisor(self, revisor="", in_ZMI=1, REQUEST=None):
      """ Delega a tarefa de revisao para revisor """
      self.revisor = revisor
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Revisor de <strong>%s</strong>: %s.' % (self.id, self.revisor)
            )

    security.declareProtected('Delegar Publicacao', 'manage_setPublicador')
    def manage_setPublicador(self, publicador="", in_ZMI=1, REQUEST=None):
      """ Delega a tarefa de publicacao para publicador """
      self.publicador = publicador
      if REQUEST is not None:
        return self.manage_WorkFlowForm(REQUEST, management_view='Workflow',
            manage_tabs_message='Novo Publicador de <strong>%s</strong>: %s.' % (self.id, self.publicador)
            )


    security.declareProtected('Gerenciar Workflow', 'getProdutores')
    def getProdutores(self, REQUEST=None):
      """ Retorna a lista de produtores """
      result = []
      uf = getattr(self, 'acl_users')
      for u in uf.getUsers():
        if u.has_role('Produtor'):
          result.append(u.getUserName())
      return result

    security.declareProtected('Gerenciar Workflow', 'getRevisores')
    def getRevisores(self, REQUEST=None):
      """ Retorna a lista de revisores """
      result = []
      uf = getattr(self, 'acl_users')
      for u in uf.getUsers():
        if u.has_role('Revisor'):
          result.append(u.getUserName())
      return result

    security.declareProtected('Gerenciar Workflow', 'getPublicadores')
    def getPublicadores(self, REQUEST=None):
      """ Retorna a lista de publicadores """
      result = []
      uf = getattr(self, 'acl_users')
      for u in uf.getUsers():
        if u.has_role('Publicador'):
          result.append(u.getUserName())
      return result

InitializeClass(XWorkFlow)
