import ExtensionClass, OFS, Globals, DateTime
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from ExtensionClass import Base
from Globals import InitializeClass, Persistent, Acquisition
from OFS.Cache import Cacheable
from OFS.Traversable import Traversable
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from utilities import dt2us
from App.Management import Navigation
from AccessControl import getSecurityManager
from DateTime import DateTime

class ZCoMIX_Common( Persistent, Base ):

    security = ClassSecurityInfo()

    security.declareProtected('View management screens', 'gadgetCustomProperties')
    manage_page_header = Globals.DTMLFile("dtml/manage_page_header", globals() )

    def checaPermissao(self, permissao):
      """ Checa se o usuario atual possui dada permissao """
      return getSecurityManager().checkPermission(permissao, self)

    def __setstate__(self, state):
        ZCoMIX_Common.inheritedAttribute('__setstate__')(self, state)
        if not hasattr(self, '_workflow_status'):
          self._workflow_status = self._valores_workflow_status['Publicado']
        if not hasattr(self, '_status'):
            self._status = 'publico'
        else:
          if not (self._status in ['publico', 'privado']):
            self._status = 'publico'
        if not hasattr(self, '_historico'):
          self._historico = []
        if hasattr(self, 'produtor') and self.produtor=="":
          self.produtor = 'admin'
        if hasattr(self, 'revisor') and self.revisor=="":
          self.revisor = 'admin'
        if hasattr(self, 'publicador') and self.publicador=="":
          self.publicador = 'admin'
        if not hasattr(self, '_catalogar'):
          self._catalogar = 1
        if hasattr(self, 'body.htm') and self.meta_type=='XPagina':
            body = getattr(self, 'body.htm')
            self._texto = body.document_src()
            self.manage_delObjects(ids='body.htm')
        if hasattr(self, 'render') and self.meta_type=='XPagina':
            body = getattr(self, 'render', None)
            if hasattr(body, 'meta_type') and body.meta_type in ['DTML Document', 'DTML Method']:
              self._texto = body.document_src()
              self.manage_delObjects(ids='render')
        if not hasattr(self, '_summary'):
          self._summary = None
        if not hasattr(self, '_keywords'):
          self._keywords = None
        if hasattr(self, '_keywords') and type(self._keywords) != type([]):
          self._keywords = []
        if not hasattr(self, 'categorias'):
          self.categorias = []
        if hasattr(self, 'categorias') and type(self.categorias) != type([]):
          self.categorias = []
        if self.meta_type == 'XSite':
          if not hasattr(self, '_noticias_capa'):
              self._noticias_capa = []
          if not hasattr(self, 'email_responsavel_site'):
            self.email_responsavel_site = "webmaster@localhost"
          self._oldManage=Navigation.manage
        if hasattr(self, '_texto'):
          self._text = self._texto
          del self._texto

    security.declareProtected('Access contents information', 'long_title')
    def long_title(self, format='html', REQUEST=None):
        "Titulo da Secao + Titulo da Pagina"
        if format == 'html':
                title = self.title_or_id()
                parent = self.aq_parent
                while hasattr(parent, 'meta_type') and parent.meta_type!='XSite':
                        if parent.meta_type=='XSecao':
                          try: title = parent.title_or_id() + "&nbsp;&raquo;&nbsp;" + title
                          except: pass
                        parent = parent.aq_parent
                return title
        title = self.aq_parent.title_or_id() + " : " + self.title_or_id()
        parent = self.aq_parent
        while parent.meta_type!='XSite':
                if parent.meta_type=='XSecao':
                        title = parent.title_or_id() + " : " + title
                parent = parent.aq_parent
        return title

    security.declareProtected('Access contents information', 'Caminho')
    def Caminho(self, REQUEST=None):
        """Caminho ate a raiz"""
        result = []
        req = None

        if REQUEST is None:
          REQUEST = self.REQUEST

        if REQUEST is not None:
                req = REQUEST

        if not req and self.REQUEST is not None:
                req = self.REQUEST

        if not req:
                return result

        i = 0;
        for node in req.PARENTS:
          try:
            if hasattr(node, 'meta_type') and (node.meta_type != 'XTemplate' or i==0):
              result.append(node)
              if node.meta_type == 'XSite':
                break
          except:
            break
          i += 1
        return result

    security.declareProtected('Access contents information', 'keywords')
    def keywords(self, REQUEST=None):
        "Resumo do Texto Secao"
        if not hasattr(self, '_keywords') or self._keywords is None:
          return []
        return self._keywords


    security.declareProtected('Access contents information','data_criacao')
    def data_criacao(self, REQUEST=None):
        """Retorna a data de criacao de um objeto"""
        if hasattr(self,'_data_criacao'):
          return self._data_criacao
        elif hasattr(self, 'data_producao'):
          if callable(self.data_producao):
            return self.data_producao()
          else:
            return self.data_producao
        else:
          self._data_criacao = DateTime()
          return self._data_criacao

    def tag(self, title=None, text=None, **args):
        """
        Gera um tag HTML HREF para este objeto, com customizacao.
        Argumentos para self.tab() podem ser quaisquer atributos validos
        de uma tag HREF.

        'href' sera sempre um caminho absoluto, para prevenir redundancia.
        """

	atributos =   { 'link_style':'style', 
			'link_class':'class',
			'link_target':'target',
			'link_onClick':'onClick',
			'link_description':'title',
			}

        result='<a href="%s"' % (self.absolute_url())

        if title is None:
            title=getattr(self, 'title', '')
	if getattr(self.aq_base, 'link_description', None) is None:
	   result = '%s title="%s"' % (result, title)

	for item in atributos.keys():
	    value = getattr(self.aq_base, item, None)
	    if value is not None:
		atributo = atributos.get(item)
            	result = '%s %s="%s"' % (result, atributo, value)		
	    else:
		if item in args.keys():
		    value = args.get(item, None)
		    if value is not None:
			atributo = atributos.get(item)
            		result = '%s %s="%s"' % (result, atributo, value)				

        if text is None:
            text = self.title_or_id()
        result = '%s>%s</a>' % (result, text)

        return result

InitializeClass(ZCoMIX_Common)

